//
//
//    ui.cpp
//
//

#include <iostream>
#include "ui.h"
#include <fstream>
using namespace std;

static bool exitRequested = false;

void exit ()
{
     exitRequested = true;
     return;
}

//----------------------------------------------------------------------
bool wasExitRequested ()
{
    return exitRequested;
}
//----------------------------------------------------------------------
void printCommandPrompt ()
{
    cout << "\n";
    cout << "-------------------------------------------------------\n";
    cout << "----    Limuzinu nuoma    -----------------------------\n";
    cout << "-------------------------------------------------------\n";
    cout << "Iveskite komanda (m - rodyti meniu): ";
}
//----------------------------------------------------------------------
string inputCommand ()
{
    string command;
    cin >> command;
    return command;
}
//----------------------------------------------------------------------
void printMenu       ();
void addLimo         (Limos&     limos);
void addCustomer     (Customers&  customers);
void rentLimo        (Limos&     limos, Customers& customers);
void returnBackLimo  (Limos&     limos);
void printRented     (Limos&     limos);
void printNotRented  (Limos&     limos);
void moneyEarnedFromCustomer (Customers& customers,Limos& limos);
void mostRented      (Limos&     limos);
void mostEarned      (Limos&     limos);
void readFromFileLimo (Limos&    limos);
//----------------------------------------------------------------------
void processCommand (Limos& limos, Customers& customers, string command)
{
    
          //printRented(limos);
         if (command == "0") exitRequested = true;
    else if (command == "m") printMenu       ();
    else if (command == "1") printLimosAndReturnCount(limos, LIMO_FILTER_ALL);
    else if (command == "2") printRented     (limos);
    else if (command == "3") printNotRented  (limos);
    else if (command == "4") addLimo         (limos);
    else if (command == "5") rentLimo        (limos, customers);
    else if (command == "6") returnBackLimo  (limos);
    else if (command == "7") printCustomersAndReturnCount(customers);
    else if (command == "8") addCustomer     (customers);
    else if (command == "9") moneyEarnedFromCustomer(customers,limos);
    else if (command =="10") mostRented(limos); 
    else if (command =="11") mostEarned(limos);
    else cout << endl << "Neteisinga komanda." << endl;
}
//----------------------------------------------------------------------
void printLimo (const Limo&  limo)
{
    cout << "id="          << limo.id
         << "\tklientas="  << limo.rentedByCustomer
         << "\t"           << limo.hourPrice << "Lt"
         << "\t(uzdirbo="  << limo.moneyEarned << "Lt)"
         << "\t"           << limo.year
         << "\t"           << limo.mark
         << "\t"           << limo.model
         << endl;
}
//----------------------------------------------------------------------
void printCustomer (const Customer& customer)
{
    cout << "id="               << customer.id
         << "\t(uzdirbo   "     << customer.moneyEarned << "Lt)"
         << "\t"                << customer.name
         << " (Asmnes kodas: "  << customer.pCode << ")"
         << endl;
}
//----------------------------------------------------------------------
int printLimosAndReturnCount (const Limos& limos, LimoFilter limoFilter)
{
    int count = 0;
    for (int i = 0;  i < limos.size();  i++)
    {
        Limo limo = limos[i];
        if (limoFilter == LIMO_FILTER_ALL
        || (limoFilter == LIMO_FILTER_RENTED      &&  isLimoRented(limo))
        || (limoFilter == LIMO_FILTER_NOT_RENTED  && !isLimoRented(limo))
           )
        {
            printLimo(limo);
            count += 1;
        }
    }
    if (count == 0)
    {
        switch (limoFilter)
        {
            case LIMO_FILTER_RENTED     : cout << "Isnuomuotu";  break;
            case LIMO_FILTER_NOT_RENTED : cout << "Laisvu";      break;
            default                     : cout << "Visu";        break;
        }
        cout << " limuzinu sarasas yra tuscias." << endl;
    }
    return count;
}
//----------------------------------------------------------------------
int printCustomersAndReturnCount (const Customers& customers)
{
    for (int i = 0;  i < customers.size();  i++)
        printCustomer( customers[i] );
    int count = customers.size();
    if (count == 0)
        cout << "Klientu sarasas yra tuscias." << endl;
    return count;
}
//----------------------------------------------------------------------
void printMenu ()
{
    cout << endl
         << "  1 - spausdinti limuzinu sarasa"     << endl
         << "  2 - spausdinti isnomuotu limuzinu sarasa" << endl
         << "  3 - spausdinti laisvu limuzinu sarasa"  << endl
         << endl
         << "  4 - prideti nauja limuzina"   << endl
         << "  5 - isnuomuoti klientui"   << endl
         << "  6 - limuzino grazinimas"      << endl
         << endl
         << "  7 - spausdinti klientus"   << endl
         << "  8 - prideti nauja klienta" << endl
         << "  9 - isnomuoti limuzinai pagal klienta" << endl
         << endl
         << " 10 - populiariausias limuzinas " << endl
         << " 11 - daugiausiai uzdirbes limuzinas" << endl
         << endl
         << "  0 - baigti darba"   << endl;
}
//----------------------------------------------------------------------
string readNonEmptyLineFromCin ()
{
    string line = "";
    while (line.size() <= 0)
        getline(cin, line);
    return line;
}
//----------------------------------------------------------------------
void addLimo (Limos& limos)
{  
    string  mark;
    string  model;
    int     year;
    string  capacity;
    int     seatsNr;
    double  hourPrice;
    cout << "  Iveskite nauja limuzina:" << endl;
    cout << "  Limuzino markes pavadinimas  : ";  mark     = readNonEmptyLineFromCin();
    cout << "  Limuzino modelio pavadinimas : ";  model    = readNonEmptyLineFromCin();
    cout << "  Gamybos metai                : ";  cin >> year;
    cout << "  Limuzino talpa(didele/maza)  : ";  capacity = readNonEmptyLineFromCin();
    cout << "  Vietu skaicius limuzine      : ";  cin >> seatsNr;
    cout << "  Kaina valandai               : ";  cin >> hourPrice;
    Limo limo = createLimo(limos.size(), mark, model, year, capacity, seatsNr, hourPrice);
    limos.push_back(limo);
}
//----------------------------------------------------------------------
void addCustomer (Customers& customers)
{
    string  name;
    int age;
    string pCode;
    cout << "Iveskite nauja klienta:" << endl;
    cout << " vardas      : ";  name    = readNonEmptyLineFromCin();
    cout << " amzius      : ";  cin >> age;
    cout << " asmens kodas: ";  cin >> pCode;  
    
    Customer customer = createCustomer(customers.size(), name, age, pCode);
    customers.push_back(customer);

}
//----------------------------------------------------------------------
int inputLimoId (const Limos& limos, LimoFilter limoFilter)
{
    int count = printLimosAndReturnCount(limos, limoFilter);
    if (count == 0)
        return NULL_ID;
    while (true)
    {
        cout << "Iveskite limuzino Id (" << NULL_ID << " - nutraukti): ";
        int id = NULL_ID;
        cin  >> id;
        if (id == NULL_ID  ||  (id >= 0  &&  id < limos.size()))
            return id;
        cout << "KLAIDA: nera limuzino su ivestu Id." << endl;
    }
}
//----------------------------------------------------------------------
int inputCustomerId (const Customers& customers)
{
    int count = printCustomersAndReturnCount(customers);
    if (count == 0)
        return NULL_ID;
    while (true)
    {
        cout << "Iveskite kliento Id (" << NULL_ID << " - nutraukti): ";
        int id = NULL_ID;
        cin  >> id;
        if (id == NULL_ID  ||  (id >= 0  &&  id < customers.size()))
            return id;
        cout << "KLAIDA: nera kliento su ivestu Id." << endl;
    }
}
//----------------------------------------------------------------------
void rentLimo (Limos& limos, Customers& customers)
{
     int limoId = inputLimoId(limos, LIMO_FILTER_NOT_RENTED);
     if (limoId == NULL_ID)
         return;
     if (isLimoRented(limos[limoId]))
     {
         cout << "KLAIDA: limuzinas jau yra isnuomotas.";
         return;
     }

     int customerId = inputCustomerId(customers);
     if (customerId == NULL_ID)
         return;

     int dayCount;
     cout << "Keliom dienom nuomuosime: ";
     cin  >> dayCount;

     double moneyEarned = rentLimoAndGetMoneyEarned(
         limos[limoId], customers[customerId], dayCount);
     cout << "Iplaukos: " << moneyEarned << " Lt" << endl;
}
//----------------------------------------------------------------------
void returnBackLimo (Limos& limos)
{
     int limoId = inputLimoId(limos, LIMO_FILTER_RENTED);
     if (limoId == NULL_ID)
         return;
     returnBackRentedLimo(limos[limoId]);
}
//=====================================================================
void printRented (Limos& limos)
{
     int count;
     count = 0;
     for (int i = 0; i < limos.size(); i++)
     {
          Limo limo = limos[i];
          if (limo.rentedByCustomer > -1)
          {
               cout << "id="          << limo.id
               << "\tklientas="  << limo.rentedByCustomer
               << "\t"           << limo.hourPrice << "Lt"
               << "\t(uzdirbo="  << limo.moneyEarned << "Lt)"
               << "\t"           << limo.year
               << "\t"           << limo.mark
               << "\t"           << limo.model
               << endl;
               count++;
          }
     }
     if (count == 0)
        //  cout << "Isnuomuotu limuzinu sarasas yra tuscias!" << endl;
     return;
}
//=====================================================================
void printNotRented (Limos& limos)
{
     int count;
     count = 0;
     for (int i = 0; i < limos.size(); i++)
     {
          Limo limo = limos[i];
          if (limo.rentedByCustomer == -1)
          {
               cout << "id="     << limo.id
               << "\tklientas="  << limo.rentedByCustomer
               << "\t"           << limo.hourPrice << "Lt"
               << "\t(uzdirbo="  << limo.moneyEarned << "Lt)"
               << "\t"           << limo.year
               << "\t"           << limo.mark
               << "\t"           << limo.model
               << endl;
               count++;
          }
     }
     if (count == 0)
          cout << "Laisvu limuzinu sarasas yra tuscias!" << endl;
     return;
}
//=====================================================================
void moneyEarnedFromCustomer (Customers& customers,Limos& limos)
{    
     int count;
     count = 0;
     int customerId = inputCustomerId(customers);    
     if (customerId == NULL_ID)
          return;
     cout << "  -------- Klientas nuomoja siuos limuzinus: ---------" << endl;
     for (int i = 0; i < limos.size(); i++)
     {
          Limo limo = limos[i];
          if (limo.rentedByCustomer == customerId)
          {
               cout << "id="     << limo.id
               << "\t"           << limo.hourPrice << "Lt"
               << "\t(uzdirbo="  << limo.moneyEarned << "Lt)"
               << "\t"           << limo.year
               << "\t"           << limo.mark
               << "\t"           << limo.model
               << endl;
               count++;
          }
     }
     if (count == 0)
          cout << "Sis klientas nenomuoja jokiu limuzinu!" << endl;
     return;
}

//=====================================================================
void mostRented (Limos& limos)
{
     int timesRented;
     int id;
     timesRented = NULL_ID;
     id   = NULL_ID;
     for (int i = 0; i < limos.size(); i++)
     {
          Limo limo = limos[i];
          if (limo.timesRented > timesRented)
          {
               timesRented = limo.timesRented;
               id          = i;
          }
     }
     if (id > NULL_ID)
     {
          Limo limo = limos[id];
          cout << " ----------- POPULIARIAUSIAS LIMUZINAS -------------------- " << endl << endl;
          cout << "id="     << limo.id
          << "\t"           << limo.hourPrice << "Lt"
          << "\t(uzdirbo="  << limo.moneyEarned << "Lt)"
          << "\t"           << limo.year
          << "\t"           << limo.mark
          << "\t"           << limo.model
          << endl;
     }
     else
     cout << "Limuzinu sarasas yra tuscias";
     return;
}
//=====================================================================
void mostEarned (Limos& limos)
{
     int moneyEarned;
     int id;
     moneyEarned = NULL_ID;
     id   = NULL_ID;
     for (int i = 0; i < limos.size(); i++)
     {
          Limo limo = limos[i];
          if (limo.moneyEarned > moneyEarned)
          {
               moneyEarned = limo.moneyEarned;
               id          = i;
          }
     }
     if (id > NULL_ID)
     {
          Limo limo = limos[id];
          cout << " ----------- DAUGIAUSIAI UZDIRBES LIMUZINAS -------------------- " << endl << endl;
          cout << "id="     << limo.id
          << "\t"           << limo.hourPrice << "Lt"
          << "\t(uzdirbo="  << limo.moneyEarned << "Lt)"
          << "\t"           << limo.year
          << "\t"           << limo.mark
          << "\t"           << limo.model
          << endl;
     }
     else
     cout << "Limuzinu sarasas yra tuscias";
     return;
}


